@@ -18,6 +18,7 @@ import (
 	"bytes"
 	"fmt"
 	"go/ast"
+	"go/constant"
 	"go/token"
 	"go/types"
 	"os"
@@ -29,12 +30,14 @@ import (
 	"strconv"
 	"strings"
 	"unicode"
+	"unicode/utf8"
 
 	"github.com/spf13/cobra"
 	"golang.org/x/tools/go/packages"
 
 	cueast "cuelang.org/go/cue/ast"
 	"cuelang.org/go/cue/ast/astutil"
+	"cuelang.org/go/cue/errors"
 	"cuelang.org/go/cue/format"
 	"cuelang.org/go/cue/literal"
 	"cuelang.org/go/cue/load"
@@ -216,6 +219,8 @@ restrictive enum interpretation of #Switch remains.
 
 	cmd.Flags().StringP(string(flagPackage), "p", "", "package name for generated CUE files")
 
+	cmd.Flags().String(string(flagOutFile), "", "generate one CUE file for a single Go package")
+
 	return cmd
 }
 
@@ -226,7 +231,7 @@ const (
 
 func (e *extractor) initExclusions(str string) {
 	e.exclude = str
-	for _, re := range strings.Split(str, ",") {
+	for re := range strings.SplitSeq(str, ",") {
 		if re != "" {
 			e.exclusions = append(e.exclusions, regexp.MustCompile(re))
 		}
@@ -249,13 +254,13 @@ type extractor struct {
 	done    map[string]bool
 
 	// per package
+	pkg      *packages.Package
 	orig     map[types.Type]*ast.StructType
 	usedPkgs map[string]bool
+	consts   map[string][]string
 
 	// per file
 	cmap     ast.CommentMap
-	pkg      *packages.Package
-	consts   map[string][]string
 	pkgNames map[string]pkgInfo
 
 	exclusions []*regexp.Regexp
@@ -355,6 +360,10 @@ var toString = []*types.Interface{
 // - consider not including types with any dropped fields.
 
 func extract(cmd *Command, args []string) error {
+	if flagLocal.IsSet(cmd) && flagOutFile.IsSet(cmd) {
+		return errors.New("--local and --outfile are mutually exclusive")
+	}
+
 	// TODO the CUE load using "." (below) assumes that a CUE module and a Go
 	// module will exist within the same directory (more precisely a Go module
 	// could be nested within a CUE module), such that the module path in any
@@ -380,19 +389,12 @@ func extract(cmd *Command, args []string) error {
 	if err != nil {
 		return err
 	}
-	var errs []string
-	for _, p := range pkgs {
-		for _, e := range p.Errors {
-			switch e.Kind {
-			case packages.ParseError, packages.TypeError:
-				// TODO: Consider making this behavior an option.
-			default:
-				errs = append(errs, fmt.Sprintf("\t%s: %v", p.PkgPath, e))
-			}
-		}
+	if packages.PrintErrors(pkgs) > 0 {
+		return ErrPrintedError
 	}
-	if len(errs) > 0 {
-		return fmt.Errorf("could not load Go packages:\n%s", strings.Join(errs, "\n"))
+
+	if flagOutFile.IsSet(cmd) && len(pkgs) != 1 {
+		return errors.New("--outfile only allows for one package to be specified")
 	}
 
 	e := extractor{
@@ -433,13 +435,12 @@ func (e *extractor) addPackage(p *packages.Package) {
 
 func (e *extractor) recordTypeInfo(p *packages.Package) {
 	for _, f := range p.Syntax {
-		ast.Inspect(f, func(n ast.Node) bool {
+		for n := range ast.Preorder(f) {
 			switch n := n.(type) {
 			case *ast.StructType:
 				e.orig[p.TypesInfo.TypeOf(n)] = n
 			}
-			return true
-		})
+		}
 	}
 }
 
@@ -472,8 +473,11 @@ func (e *extractor) extractPkg(root string, p *packages.Package) error {
 		}
 	}
 
-	if err := os.MkdirAll(dir, 0777); err != nil {
-		return err
+	outFile := flagOutFile.String(e.cmd)
+	if outFile == "" {
+		if err := os.MkdirAll(dir, 0777); err != nil {
+			return err
+		}
 	}
 
 	e.usedPkgs = map[string]bool{}
@@ -483,6 +487,15 @@ func (e *extractor) extractPkg(root string, p *packages.Package) error {
 		args += " --exclude=" + e.exclude
 	}
 
+	pName := flagPackage.String(e.cmd)
+	if pName == "" {
+		pName = p.Name
+	}
+
+	cueFile, cuePkg := e.newFile(pName, args)
+	// By default, we output one CUE file for each Go file as long as there's anything to generate.
+	// When --outfile is used, we want to generate exactly one CUE file for an entire Go package,
+	// so we instead keep joining declarations until we reach the last Go file, where we then write.
 	for i, f := range p.Syntax {
 		e.cmap = ast.NewCommentMap(p.Fset, f, f.Comments)
 
@@ -504,53 +517,51 @@ func (e *extractor) extractPkg(root string, p *packages.Package) error {
 			e.pkgNames[pkgPath] = info
 		}
 
-		decls := []cueast.Decl{}
+		addDoc(f.Doc, cuePkg)
 		for _, d := range f.Decls {
 			switch d := d.(type) {
 			case *ast.GenDecl:
-				decls = append(decls, e.reportDecl(d)...)
+				cueFile.Decls = append(cueFile.Decls, e.reportDecl(d)...)
 			}
 		}
 
-		if len(decls) == 0 && f.Doc == nil {
+		if outFile == "" && len(cueFile.Decls) == 0 && f.Doc == nil {
+			// By default, empty files are not generated.
 			continue
 		}
 
-		pName := flagPackage.String(e.cmd)
-		if pName == "" {
-			pName = p.Name
+		if outFile != "" && i != len(p.Syntax)-1 {
+			// By default, we generate one CUE file per Go file.
+			// With --outfile, we keep adding to one output file, and write it at the end.
+			// TODO: using a second loop to write output files may be simpler.
+			continue
 		}
 
-		pkg := &cueast.Package{Name: e.ident(pName, false)}
-		addDoc(f.Doc, pkg)
-
-		f := &cueast.File{Decls: []cueast.Decl{
-			&cueast.CommentGroup{List: []*cueast.Comment{
-				{Text: "// Code generated by cue get go. DO NOT EDIT."},
-			}},
-			&cueast.CommentGroup{List: []*cueast.Comment{
-				{Text: "//cue:generate cue get go " + args},
-			}},
-			pkg,
-		}}
-		f.Decls = append(f.Decls, decls...)
-
-		if err := astutil.Sanitize(f); err != nil {
+		if err := astutil.Sanitize(cueFile); err != nil {
 			return err
 		}
 
-		file := filepath.Base(p.CompiledGoFiles[i])
-
-		file = strings.Replace(file, ".go", "_go", 1)
-		file += "_gen.cue"
-		b, err := format.Node(f, format.Simplify())
+		b, err := format.Node(cueFile, format.Simplify())
 		if err != nil {
 			return err
 		}
-		err = os.WriteFile(filepath.Join(dir, file), b, 0666)
+
+		dst := outFile
+		if dst == "" {
+			file := filepath.Base(p.CompiledGoFiles[i])
+			file = strings.Replace(file, ".go", "_go", 1)
+			file += "_gen.cue"
+			dst = filepath.Join(dir, file)
+		}
+		if dst == "-" {
+			_, err = os.Stdout.Write(b)
+		} else {
+			err = os.WriteFile(dst, b, 0666)
+		}
 		if err != nil {
 			return err
 		}
+		cueFile, cuePkg = e.newFile(pName, args)
 	}
 
 	if !isMain {
@@ -559,11 +570,14 @@ func (e *extractor) extractPkg(root string, p *packages.Package) error {
 		}
 	}
 
-	for path := range e.usedPkgs {
-		if !e.done[path] {
-			e.done[path] = true
-			if err := e.extractPkg(root, e.allPkgs[path]); err != nil {
-				return err
+	if outFile == "" {
+		// Note that we don't generate dependencies with --outfile.
+		for pkgPath := range e.usedPkgs {
+			if !e.done[pkgPath] {
+				e.done[pkgPath] = true
+				if err := e.extractPkg(root, e.allPkgs[pkgPath]); err != nil {
+					return err
+				}
 			}
 		}
 	}
@@ -571,6 +585,20 @@ func (e *extractor) extractPkg(root string, p *packages.Package) error {
 	return nil
 }
 
+func (e *extractor) newFile(pName, args string) (*cueast.File, *cueast.Package) {
+	pkg := &cueast.Package{Name: e.ident(pName, false)}
+	file := &cueast.File{Decls: []cueast.Decl{
+		&cueast.CommentGroup{List: []*cueast.Comment{
+			{Text: "// Code generated by cue get go. DO NOT EDIT."},
+		}},
+		&cueast.CommentGroup{List: []*cueast.Comment{
+			{Text: "//cue:generate cue get go " + args},
+		}},
+		pkg,
+	}}
+	return file, pkg
+}
+
 func (e *extractor) importCUEFiles(p *packages.Package, dstDir, args string) error {
 	// A package's Go files will often share directories,
 	// for example all original source files will sit in the same directory.
@@ -595,7 +623,7 @@ func (e *extractor) importCUEFiles(p *packages.Package, dstDir, args string) err
 			path := filepath.Join(srcDir, name)
 			name, ok := strings.CutSuffix(name, ".cue")
 			if !ok {
-				return nil
+				continue
 			}
 			f, err := parser.ParseFile(path, nil, parser.PackageClauseOnly)
 			if err != nil {
@@ -637,13 +665,9 @@ func (e *extractor) recordConsts(x *ast.GenDecl) {
 	}
 }
 
-func (e *extractor) strLabel(name string) cueast.Label {
-	return cueast.NewString(name)
-}
-
 func (e *extractor) ident(name string, isDef bool) *cueast.Ident {
 	if isDef {
-		r := []rune(name)[0]
+		r, _ := utf8.DecodeRuneInString(name)
 		name = "#" + name
 		if !unicode.Is(unicode.Lu, r) {
 			name = "_" + name
@@ -788,11 +812,30 @@ func (e *extractor) reportDecl(x *ast.GenDecl) (a []cueast.Decl) {
 				}
 
 				typ := e.pkg.TypesInfo.TypeOf(name)
-				c := e.pkg.TypesInfo.Defs[v.Names[i]].(*types.Const)
-				sv := c.Val().ExactString()
-				cv, err := parser.ParseExpr("", sv)
-				if err != nil {
-					panic(fmt.Errorf("failed to parse %v: %v", sv, err))
+				v := e.pkg.TypesInfo.Defs[v.Names[i]].(*types.Const).Val()
+				var cv cueast.Expr
+				switch v.Kind() {
+				case constant.String:
+					s := constant.StringVal(v)
+					bl := &cueast.BasicLit{Kind: cuetoken.STRING}
+					// Go strings may contain any bytes, even invalid UTF-8.
+					// CUE strings may only contain valid UTF-8, because it has
+					// bytes values for anything that may not be valid UTF-8.
+					if utf8.ValidString(s) {
+						bl.Value = literal.String.Quote(s)
+					} else {
+						bl.Value = literal.Bytes.Quote(s)
+					}
+					cv = bl
+
+				default:
+					// TODO(mvdan): replace this with switch cases for Bool/Int/Float
+					sv := v.ExactString()
+					var err error
+					cv, err = parser.ParseExpr("", sv)
+					if err != nil {
+						panic(fmt.Errorf("failed to parse %v: %v", sv, err))
+					}
 				}
 
 				// Use the original Go value if compatible with CUE (octal is okay)
@@ -855,13 +898,11 @@ func (e *extractor) altType(typ types.Type) cueast.Expr {
 	return nil
 }
 
-func addDoc(g *ast.CommentGroup, x cueast.Node) bool {
+func addDoc(g *ast.CommentGroup, x cueast.Node) {
 	doc := makeDoc(g, true)
 	if doc != nil {
 		cueast.AddComment(x, doc)
-		return true
 	}
-	return false
 }
 
 func makeDoc(g *ast.CommentGroup, isDoc bool) *cueast.CommentGroup {
@@ -878,7 +919,7 @@ func makeDoc(g *ast.CommentGroup, isDoc bool) *cueast.CommentGroup {
 		// The parser has given us exactly the comment text.
 		switch c[1] {
 		case '/':
-			//-style comment (no newline at the end)
+			// //-style comment (no newline at the end)
 			a = append(a, &cueast.Comment{Text: c})
 
 		case '*':
@@ -928,10 +969,8 @@ func makeDoc(g *ast.CommentGroup, isDoc bool) *cueast.CommentGroup {
 
 func supportedType(stack []types.Type, t types.Type) (ok bool) {
 	// handle recursive types
-	for _, t0 := range stack {
-		if t0 == t {
-			return true
-		}
+	if slices.Contains(stack, t) {
+		return true
 	}
 	stack = append(stack, t)
 
@@ -1011,7 +1050,7 @@ func (e *extractor) makeField(name string, kind fieldKind, expr types.Type, doc
 	if kind == definition {
 		label = e.ident(name, true)
 	} else {
-		label = e.strLabel(name)
+		label = cueast.NewStringLabel(name)
 	}
 	f = &cueast.Field{Label: label, Value: typ}
 	if doc := makeDoc(doc, newline); doc != nil {
@@ -1033,7 +1072,7 @@ func (e *extractor) makeField(name string, kind fieldKind, expr types.Type, doc
 
 	internal.SetConstraint(f, tok)
 
-	b, _ := format.Node(typ)
+	b, _ := format.Node(typ, format.Simplify())
 	return f, string(b)
 }
 
@@ -1041,13 +1080,14 @@ func (e *extractor) makeType(typ types.Type) (result cueast.Expr) {
 	switch typ := types.Unalias(typ).(type) {
 	case *types.Named:
 		obj := typ.Obj()
-		if obj.Pkg() == nil {
+		pkg := obj.Pkg()
+		if pkg == nil {
 			return e.ident("_", false)
 		}
 		// Check for builtin packages.
 		switch {
-		case obj.Pkg().Path() == "time" && obj.Name() == "Time":
-			ref := e.ident(e.pkgNames[obj.Pkg().Path()].name, false)
+		case pkg.Path() == "time" && obj.Name() == "Time":
+			ref := e.ident(e.pkgNames[pkg.Path()].name, false)
 			var name *cueast.Ident
 			if ref.Name != "time" {
 				name = e.ident(ref.Name, false)
@@ -1055,7 +1095,7 @@ func (e *extractor) makeType(typ types.Type) (result cueast.Expr) {
 			ref.Node = cueast.NewImport(name, "time")
 			return cueast.NewSel(ref, obj.Name())
 
-		case obj.Pkg().Path() == "time" && obj.Name() == "Duration":
+		case pkg.Path() == "time" && obj.Name() == "Duration":
 			// Go's time.Duration is an int64 to represent nanoseconds,
 			// and even though most Go users would find the string representation
 			// like "3s" or "40m" most reasonable and readable for constant values,
@@ -1073,22 +1113,36 @@ func (e *extractor) makeType(typ types.Type) (result cueast.Expr) {
 			// and constants like '300 | *"300ns"' to support both at the same time?
 			return e.ident("int", false)
 
-		case obj.Pkg().Path() == "math/big" && obj.Name() == "Int":
+		case pkg.Path() == "math/big" && obj.Name() == "Int":
 			return e.ident("int", false)
 
 		default:
-			if !strings.ContainsAny(obj.Pkg().Path(), ".") {
-				// Drop any standard library type if they haven't been handled
-				// above.
-				// TODO: Doc?
+			// Any Go standard library type that hasn't been handled above is not supported;
+			// fall back to whatever alternative type we can, or just "top".
+			// Otherwise we would end up generating "std" packages which clash with our
+			// own standard library, for example cue.mod/gen/time.
+			//
+			// Go defines standard library as "no dot in the first path element",
+			// such that "foo/bar.baz" is still technically part of the standard library.
+			// Note that "example" and "test" are reserved for users in Go, such that "test/foo"
+			// is not part of Go's standard library, but such exceptions do not exist in CUE.
+			// TODO: never place such packages under e.g. cue.mod/gen/example, as that clashes
+			// with CUE's own standard library namespace at the moment.
+			//
+			// TODO: for cases where the Go std type could be supported, we could still generate
+			// and import it under a non-std CUE package, such as cue.mod/gen/pkg.go.dev/time.
+			// TODO: Doc?
+			firstElem, _, _ := strings.Cut(pkg.Path(), "/")
+			if !strings.ContainsAny(firstElem, ".") {
 				if s := e.altType(obj.Type()); s != nil {
 					return s
 				}
+				return e.ident("_", false)
 			}
 		}
 
 		result = e.ident(obj.Name(), true)
-		if pkg := obj.Pkg(); pkg != nil && pkg != e.pkg.Types {
+		if pkg != e.pkg.Types {
 			info := e.pkgNames[pkg.Path()]
 			if info.name == "" {
 				info.name = pkg.Name()
@@ -1228,8 +1282,8 @@ func (e *extractor) makeType(typ types.Type) (result cueast.Expr) {
 
 	case *types.Union:
 		var exprs []cueast.Expr
-		for i := 0; i < typ.Len(); i++ {
-			exprs = append(exprs, e.makeType(typ.Term(i).Type()))
+		for term := range typ.Terms() {
+			exprs = append(exprs, e.makeType(term.Type()))
 		}
 		return cueast.NewBinExpr(cuetoken.OR, exprs...)
 
@@ -1432,15 +1486,18 @@ func (e *extractor) isOptional(f *types.Var, doc *ast.CommentGroup, tag string)
 		return true
 	}
 
-	for _, line := range strings.Split(doc.Text(), "\n") {
+	for line := range strings.SplitSeq(doc.Text(), "\n") {
 		before, _, _ := strings.Cut(strings.TrimSpace(line), "=")
 		if before == "+optional" {
 			return true
 		}
 	}
 
+	// Go 1.24 added the "omitzero" option to encoding/json, an improvement over "omitempty".
+	// Note that, as of mid 2025, YAML libraries don't seem to have picked up "omitzero" yet.
 	// TODO: also when the type is a list or other kind of pointer.
 	return hasFlag(tag, "json", "omitempty", 1) ||
+		hasFlag(tag, "json", "omitzero", 1) ||
 		hasFlag(tag, "yaml", "omitempty", 1)
 }
 
@@ -1450,10 +1507,8 @@ func hasFlag(tag, key, flag string, offset int) bool {
 		if offset >= len(split) {
 			return false
 		}
-		for _, str := range split[offset:] {
-			if str == flag {
-				return true
-			}
+		if slices.Contains(split[offset:], flag) {
+			return true
 		}
 	}
 	return false
@@ -1463,9 +1518,7 @@ func getName(name string, tag string) string {
 	tags := reflect.StructTag(tag)
 	for _, s := range []string{"json", "yaml"} {
 		if tag, ok := tags.Lookup(s); ok {
-			if p := strings.Index(tag, ","); p >= 0 {
-				tag = tag[:p]
-			}
+			tag, _, _ = strings.Cut(tag, ",")
 			if tag != "" {
 				return tag
 			}
