@@ -30,13 +30,17 @@ import (
 	"cuelang.org/go/internal/core/runtime"
 )
 
-// A Context is used for creating CUE [Value]s.
+// A Context is used for creating CUE [Value] objects.
 //
 // A Context keeps track of loaded instances, indices of internal
 // representations of values, and defines the set of supported builtins. Any
 // operation that involves two Values should originate from the same Context.
 //
 // Use [cuelang.org/go/cue/cuecontext.New] to create a new context.
+//
+// Note that a context may grow in size as more values are created or loaded.
+// If memory usage becomes a problem, consider avoiding long-lived contexts,
+// such as using one context per task or periodically re-creating the context.
 type Context runtime.Runtime
 
 func (c *Context) runtime() *runtime.Runtime {
@@ -369,18 +373,12 @@ func (c *Context) Encode(x interface{}, option ...EncodeOption) Value {
 	ctx := c.ctx()
 	// TODO: is true the right default?
 	expr := convert.GoValueToValue(ctx, x, options.nilIsTop)
-	var n *adt.Vertex
-	if v, ok := expr.(*adt.Vertex); ok {
-		n = v
-	} else {
-		n = &adt.Vertex{}
-		n.AddConjunct(adt.MakeRootConjunct(nil, expr))
-	}
+	n := exprToVertex(expr)
 	n.Finalize(ctx)
 	return c.make(n)
 }
 
-// Encode converts a Go type to a CUE [Value].
+// EncodeType converts a Go type to a CUE [Value].
 //
 // The returned value will represent an error, accessible through [Value.Err],
 // if any error occurred.
@@ -395,13 +393,7 @@ func (c *Context) EncodeType(x interface{}, option ...EncodeOption) Value {
 	if err != nil {
 		return c.makeError(err)
 	}
-	var n *adt.Vertex
-	if v, ok := expr.(*adt.Vertex); ok {
-		n = v
-	} else {
-		n = &adt.Vertex{}
-		n.AddConjunct(adt.MakeRootConjunct(nil, expr))
-	}
+	n := exprToVertex(expr)
 	n.Finalize(ctx)
 	return c.make(n)
 }
@@ -472,7 +464,7 @@ func str(c *adt.OpContext, v adt.Node) string {
 
 // eval returns the evaluated value. This may not be the vertex.
 //
-// Deprecated: use ctx.value
+// Deprecated: use [adt.OpContext.value].
 func (v Value) eval(ctx *adt.OpContext) adt.Value {
 	if v.v == nil {
 		panic("undefined value")
