@@ -17,14 +17,13 @@ package cue_test
 import (
 	"fmt"
 	"math/big"
-	"math/bits"
 	"reflect"
-	"strconv"
 	"testing"
 	"time"
 
 	"cuelang.org/go/cue"
 	"cuelang.org/go/internal/cuetdtest"
+	"github.com/cockroachdb/apd/v3"
 	"github.com/go-quicktest/qt"
 	"github.com/google/go-cmp/cmp"
 )
@@ -130,7 +129,7 @@ func TestDecode(t *testing.T) {
 		value: `{a:1,m:{a: 3}}`,
 		dst:   &fields{},
 		want: fields{A: 1,
-			M: map[string]interface{}{"a": int(3)}},
+			M: map[string]interface{}{"a": int64(3)}},
 	}, {
 		// indirect int
 		value: `{p: 1}`,
@@ -193,12 +192,12 @@ func TestDecode(t *testing.T) {
 		value: `{a: 1, b: 2, c: true, d: e: 2}`,
 		dst:   &map[string]interface{}{},
 		want: map[string]interface{}{
-			"a": 1, "b": 2, "c": true,
-			"d": map[string]interface{}{"e": 2}},
+			"a": int64(1), "b": int64(2), "c": true,
+			"d": map[string]interface{}{"e": int64(2)}},
 	}, {
 		value: `{a: b: *2 | int}`,
 		dst:   &map[string]interface{}{},
-		want:  map[string]interface{}{"a": map[string]interface{}{"b": int(2)}},
+		want:  map[string]interface{}{"a": map[string]interface{}{"b": int64(2)}},
 	}, {
 		value: `{a: 1, b: 2, c: true}`,
 		dst:   &map[string]int{},
@@ -217,7 +216,7 @@ func TestDecode(t *testing.T) {
 		dst:   &map[string]interface{}{},
 		want: map[string]interface{}{
 			"a": map[string]interface{}{
-				"b": []interface{}{int(0)},
+				"b": []interface{}{int64(0)},
 			},
 		},
 	}, {
@@ -246,12 +245,12 @@ func TestDecode(t *testing.T) {
 		// large integer which doesn't fit into an int32 or int on 32-bit platforms
 		value: `8000000000`,
 		dst:   new(interface{}),
-		want:  intOver32("8000000000"),
+		want:  int64(8000000000),
 	}, {
 		// same as the above, but negative
 		value: `-8000000000`,
 		dst:   new(interface{}),
-		want:  intOver32("-8000000000"),
+		want:  int64(-8000000000),
 	}, {
 		// even larger integer which doesn't fit into an int64
 		value: `9500000000000000000`,
@@ -262,23 +261,45 @@ func TestDecode(t *testing.T) {
 		value: `-9500000000000000000`,
 		dst:   new(interface{}),
 		want:  bigInt("-9500000000000000000"),
+	}, {
+		value: `9500000000000000000`,
+		dst:   new(*big.Int),
+		want:  bigInt("9500000000000000000"),
+	}, {
+		value: `-9500000000000000000`,
+		dst:   new(*big.Int),
+		want:  bigInt("-9500000000000000000"),
+	}, {
+		// TODO: should this work?
+		value: `9500000000000000000`,
+		dst:   new(*apd.Decimal),
+		err:   "Decode: cannot use value 9500000000000000000 (type int) as (string|bytes)",
+	}, {
+		// TODO: should this work?
+		value: `-9500000000000000000`,
+		dst:   new(*apd.Decimal),
+		err:   "Decode: cannot use value -9500000000000000000 (type int) as (string|bytes)",
 	}, {
 		// large float which doesn't fit into a float32
 		value: `1.797693134e+308`,
 		dst:   new(interface{}),
 		want:  float64(1.797693134e+308),
 	}, {
-		// even larger float which doesn't fit into a float64
-		// TODO(mvdan): this should work via *big.Float, just like we do with *big.Int above.
 		value: `1.99769313499e+508`,
 		dst:   new(interface{}),
-		err:   "value was rounded up",
+		want:  bigFloat(`1.99769313499e+508`),
 	}, {
-		// same as the above, but negative
-		// TODO(mvdan): this should work via *big.Float, just like we do with *big.Int above.
 		value: `-1.99769313499e+508`,
 		dst:   new(interface{}),
-		err:   "value was rounded down",
+		want:  bigFloat(`-1.99769313499e+508`),
+	}, {
+		value: `1.99769313499e+508`,
+		dst:   new(*big.Float),
+		want:  bigFloat(`1.99769313499e+508`),
+	}, {
+		value: `-1.99769313499e+508`,
+		dst:   new(*big.Float),
+		want:  bigFloat(`-1.99769313499e+508`),
 	}}
 	for _, tc := range testCases {
 		cuetdtest.FullMatrix.Run(t, tc.value, func(t *testing.T, m *cuetdtest.M) {
@@ -288,6 +309,8 @@ func TestDecode(t *testing.T) {
 			got := reflect.ValueOf(tc.dst).Elem().Interface()
 			if diff := cmp.Diff(got, tc.want, cmp.Comparer(func(a, b *big.Int) bool {
 				return a.Cmp(b) == 0
+			}), cmp.Comparer(func(a, b *big.Float) bool {
+				return a.Cmp(b) == 0
 			})); diff != "" {
 				t.Error(diff)
 				t.Errorf("\n%#v\n%#v", got, tc.want)
@@ -296,20 +319,17 @@ func TestDecode(t *testing.T) {
 	}
 }
 
-func intOver32(s string) any {
-	if bits.UintSize == 32 {
-		return bigInt(s)
-	}
-	// Work around "constant overflows int" typechecking errors on 32 bits.
-	n, _ := strconv.Atoi(s)
-	return n
-}
-
 func bigInt(s string) *big.Int {
 	n, _ := big.NewInt(0).SetString(s, 10)
 	return n
 }
 
+func bigFloat(s string) *big.Float {
+	f := &big.Float{}
+	f, _, _ = f.Parse(s, 0)
+	return f
+}
+
 func TestDecodeIntoCUEValue(t *testing.T) {
 	cuetdtest.FullMatrix.Do(t, func(t *testing.T, m *cuetdtest.M) {
 		// We should be able to decode into a CUE value so we can
