@@ -31,19 +31,10 @@ import (
 	"cuelang.org/go/cue/cuecontext"
 	"cuelang.org/go/cue/errors"
 	"cuelang.org/go/cue/format"
-	"cuelang.org/go/internal/cueexperiment"
 	"cuelang.org/go/internal/tdtest"
 )
 
 func init() {
-	// Ignore the value of CUE_EXPERIMENT for the purposes of these tests,
-	// as we want them to start off with the default experiment values.
-	os.Setenv("CUE_EXPERIMENT", "")
-
-	// Once we've called cueexperiment.Init, cueexperiment.Vars
-	// will not be touched again, so we can set fields in it for the tests.
-	cueexperiment.Init()
-
 	// The user running `go test` might have a broken environment,
 	// such as an invalid $CUE_REGISTRY like the one below,
 	// or a broken $DOCKER_CONFIG/config.json due to syntax errors.
@@ -105,7 +96,8 @@ display:.
 files:
     $CWD/testdata/testmod/test.cue
 imports:
-    mod.test/test/sub: $CWD/testdata/testmod/sub/sub.cue`}, {
+    mod.test/test/sub: $CWD/testdata/testmod/sub/sub.cue`,
+	}, {
 		name: "DefaultPackageWithExplicitDotArgument",
 		// Even though the directory is called testdata, the last path in
 		// the module is test. So "package test" is correctly the default
@@ -120,7 +112,8 @@ display:.
 files:
     $CWD/testdata/testmod/test.cue
 imports:
-    mod.test/test/sub: $CWD/testdata/testmod/sub/sub.cue`}, {
+    mod.test/test/sub: $CWD/testdata/testmod/sub/sub.cue`,
+	}, {
 		name: "RelativeImportPathWildcard",
 		cfg:  dirCfg,
 		args: []string{"./other/..."},
@@ -130,17 +123,19 @@ path:   ""
 module: mod.test/test@v0
 root:   $CWD/testdata/testmod
 dir:    ""
-display:""`}, {
+display:""`,
+	}, {
 		name: "NoMatchingPackageName",
 		cfg:  dirCfg,
 		args: []string{"./anon"},
 		want: `err:    build constraints exclude all CUE files in ./anon:
-    anon/anon.cue: no package name
+    $CWD/testdata/testmod/anon/anon.cue: no package name
 path:   mod.test/test/anon@v0
 module: mod.test/test@v0
 root:   $CWD/testdata/testmod
 dir:    $CWD/testdata/testmod/anon
-display:./anon`}, {
+display:./anon`,
+	}, {
 		name: "RelativeImportPathSingle",
 		cfg:  dirCfg,
 		args: []string{"./other"},
@@ -152,7 +147,8 @@ root:   $CWD/testdata/testmod
 dir:    $CWD/testdata/testmod/other
 display:./other
 files:
-    $CWD/testdata/testmod/other/main.cue`}, {
+    $CWD/testdata/testmod/other/main.cue`,
+	}, {
 		name: "RelativePathSuccess",
 		cfg:  dirCfg,
 		args: []string{"./hello"},
@@ -165,7 +161,8 @@ files:
     $CWD/testdata/testmod/test.cue
     $CWD/testdata/testmod/hello/test.cue
 imports:
-    mod.test/test/sub: $CWD/testdata/testmod/sub/sub.cue`}, {
+    mod.test/test/sub: $CWD/testdata/testmod/sub/sub.cue`,
+	}, {
 		name: "ExplicitPackageIdentifier",
 		cfg:  dirCfg,
 		args: []string{"mod.test/test/hello:test"},
@@ -178,14 +175,15 @@ files:
     $CWD/testdata/testmod/test.cue
     $CWD/testdata/testmod/hello/test.cue
 imports:
-    mod.test/test/sub: $CWD/testdata/testmod/sub/sub.cue`}, {
+    mod.test/test/sub: $CWD/testdata/testmod/sub/sub.cue`,
+	}, {
 		name: "NoPackageName",
 		cfg:  dirCfg,
 		args: []string{"mod.test/test/hello:nonexist"},
 		want: `err:    cannot find package "mod.test/test/hello": no files in package directory with package name "nonexist"
 path:   mod.test/test/hello:nonexist
 module: ""
-root:   $CWD/testdata/testmod
+root:   ""
 dir:    ""
 display:mod.test/test/hello:nonexist`,
 	}, {
@@ -211,7 +209,8 @@ root:   $CWD/testdata/testmod
 dir:    $CWD/testdata/testmod
 display:command-line-arguments
 files:
-    $CWD/testdata/testmod/anon.cue`}, {
+    $CWD/testdata/testmod/anon.cue`,
+	}, {
 		name: "StandardInput",
 		cfg:  dirCfg,
 		args: []string{"-"},
@@ -221,7 +220,8 @@ root:   $CWD/testdata/testmod
 dir:    $CWD/testdata/testmod
 display:command-line-arguments
 files:
-    -`}, {
+    -`,
+	}, {
 		name: "BadIdentifier",
 		cfg:  dirCfg,
 		args: []string{"foo.com/bad-identifier"},
@@ -229,7 +229,7 @@ files:
 cannot find package "foo.com/bad-identifier": cannot find module providing package foo.com/bad-identifier
 path:   foo.com/bad-identifier
 module: ""
-root:   $CWD/testdata/testmod
+root:   ""
 dir:    ""
 display:foo.com/bad-identifier`,
 	}, {
@@ -239,7 +239,7 @@ display:foo.com/bad-identifier`,
 		want: `err:    standard library import path "nonexisting" cannot be imported as a CUE package
 path:   nonexisting
 module: ""
-root:   $CWD/testdata/testmod
+root:   ""
 dir:    ""
 display:nonexisting`,
 	}, {
@@ -249,7 +249,7 @@ display:nonexisting`,
 		want: `err:    standard library import path "strconv" cannot be imported as a CUE package
 path:   strconv
 module: ""
-root:   $CWD/testdata/testmod
+root:   ""
 dir:    ""
 display:strconv`,
 	}, {
@@ -275,7 +275,8 @@ files:
     $CWD/testdata/testmod/imports/imports.cue
 imports:
     mod.test/catch: $CWD/testdata/testmod/cue.mod/pkg/mod.test/catch/catch.cue
-    mod.test/helper:helper1: $CWD/testdata/testmod/cue.mod/pkg/mod.test/helper/helper1.cue`}, {
+    mod.test/helper:helper1: $CWD/testdata/testmod/cue.mod/pkg/mod.test/helper/helper1.cue`,
+	}, {
 		name: "PackageWithImportsWithSkipImportsConfig",
 		cfg: &Config{
 			Dir:         testdataDir,
@@ -289,7 +290,8 @@ root:   $CWD/testdata/testmod
 dir:    $CWD/testdata/testmod/imports
 display:./imports
 files:
-    $CWD/testdata/testmod/imports/imports.cue`}, {
+    $CWD/testdata/testmod/imports/imports.cue`,
+	}, {
 		name: "OnlyToolFiles",
 		cfg:  dirCfg,
 		args: []string{"./toolonly"},
@@ -299,20 +301,36 @@ root:   $CWD/testdata/testmod
 dir:    $CWD/testdata/testmod/toolonly
 display:./toolonly
 files:
-    $CWD/testdata/testmod/toolonly/foo_tool.cue`}, {
+    $CWD/testdata/testmod/toolonly/foo_tool.cue`,
+	}, {
 		name: "OnlyToolFilesWithToolsDisabledInConfig",
 		cfg: &Config{
 			Dir: testdataDir,
 		},
 		args: []string{"./toolonly"},
 		want: `err:    build constraints exclude all CUE files in ./toolonly:
-    test.cue: package is test, want foo
-    toolonly/foo_tool.cue: _tool.cue files excluded in non-cmd mode
+    $CWD/testdata/testmod/test.cue: package is test, want foo
+    $CWD/testdata/testmod/toolonly/foo_tool.cue: _tool.cue files excluded in non-cmd mode
 path:   mod.test/test/toolonly@v0:foo
 module: mod.test/test@v0
 root:   $CWD/testdata/testmod
 dir:    $CWD/testdata/testmod/toolonly
-display:./toolonly`}, {
+display:./toolonly`,
+	}, {
+		name: "OnlyTestFilesWithTestsDisabledInConfig",
+		cfg: &Config{
+			Dir: testdataDir,
+		},
+		args: []string{"./testonly"},
+		want: `err:    build constraints exclude all CUE files in ./testonly:
+    $CWD/testdata/testmod/test.cue: package is test, want foo
+    $CWD/testdata/testmod/testonly/foo_test.cue: _test.cue files excluded in non-test mode
+path:   mod.test/test/testonly@v0:foo
+module: mod.test/test@v0
+root:   $CWD/testdata/testmod
+dir:    $CWD/testdata/testmod/testonly
+display:./testonly`,
+	}, {
 		name: "WithBoolTag",
 		cfg: &Config{
 			Dir:  testdataDir,
@@ -325,7 +343,8 @@ root:   $CWD/testdata/testmod
 dir:    $CWD/testdata/testmod/tags
 display:./tags
 files:
-    $CWD/testdata/testmod/tags/prod.cue`}, {
+    $CWD/testdata/testmod/tags/prod.cue`,
+	}, {
 		name: "WithAttrValTag",
 		cfg: &Config{
 			Dir:  testdataDir,
@@ -338,7 +357,8 @@ root:   $CWD/testdata/testmod
 dir:    $CWD/testdata/testmod/tags
 display:./tags
 files:
-    $CWD/testdata/testmod/tags/prod.cue`}, {
+    $CWD/testdata/testmod/tags/prod.cue`,
+	}, {
 		name: "UnusedTag",
 		cfg: &Config{
 			Dir:  testdataDir,
@@ -354,7 +374,8 @@ path:   mod.test/test/tagsbad@v0
 module: mod.test/test@v0
 root:   $CWD/testdata/testmod
 dir:    $CWD/testdata/testmod/tagsbad
-display:./tagsbad`}, {
+display:./tagsbad`,
+	}, {
 		name: "ImportCycle",
 		cfg: &Config{
 			Dir: testdataDir,
@@ -370,7 +391,8 @@ root:   $CWD/testdata/testmod
 dir:    $CWD/testdata/testmod/cycle
 display:./cycle
 files:
-    $CWD/testdata/testmod/cycle/cycle.cue`}, {
+    $CWD/testdata/testmod/cycle/cycle.cue`,
+	}, {
 		name: "AcceptLegacyModuleWithLegacyModule",
 		cfg: &Config{
 			Dir:                 testdata("testmod_legacy"),
@@ -382,7 +404,8 @@ root:   $CWD/testdata/testmod_legacy
 dir:    $CWD/testdata/testmod_legacy
 display:.
 files:
-    $CWD/testdata/testmod_legacy/foo.cue`}, {
+    $CWD/testdata/testmod_legacy/foo.cue`,
+	}, {
 		name: "AcceptLegacyModuleWithNonLegacyModule",
 		cfg: &Config{
 			Dir:                 testdataDir,
@@ -399,7 +422,8 @@ files:
     $CWD/testdata/testmod/imports/imports.cue
 imports:
     mod.test/catch: $CWD/testdata/testmod/cue.mod/pkg/mod.test/catch/catch.cue
-    mod.test/helper:helper1: $CWD/testdata/testmod/cue.mod/pkg/mod.test/helper/helper1.cue`}, {
+    mod.test/helper:helper1: $CWD/testdata/testmod/cue.mod/pkg/mod.test/helper/helper1.cue`,
+	}, {
 		name: "MismatchedModulePathInConfig",
 		cfg: &Config{
 			Dir:    testdataDir,
@@ -412,7 +436,8 @@ path:   ""
 module: wrong.test@v0
 root:   ""
 dir:    ""
-display:""`}, {
+display:""`,
+	}, {
 		name: "ModulePathInConfigWithoutMajorVersion",
 		cfg: &Config{
 			Dir:    testdataDir,
@@ -425,7 +450,8 @@ path:   ""
 module: mod.test/test
 root:   ""
 dir:    ""
-display:""`}, {
+display:""`,
+	}, {
 		name: "ModulePathInConfigWithoutMajorVersionAndMismatchedPath",
 		cfg: &Config{
 			Dir:    testdataDir,
@@ -438,7 +464,8 @@ path:   ""
 module: mod.test/wrong
 root:   ""
 dir:    ""
-display:""`}, {
+display:""`,
+	}, {
 		name: "ExplicitPackageWithUnqualifiedImportPath#1",
 		cfg: &Config{
 			Dir:     filepath.Join(testdataDir, "multi"),
@@ -451,7 +478,8 @@ root:   $CWD/testdata/testmod
 dir:    $CWD/testdata/testmod/multi
 display:.
 files:
-    $CWD/testdata/testmod/multi/file.cue`}, {
+    $CWD/testdata/testmod/multi/file.cue`,
+	}, {
 		name: "ExplicitPackageWithUnqualifiedImportPath#2",
 		// This test replicates the failure reported in https://cuelang.org/issue/3213
 		cfg: &Config{
@@ -467,7 +495,8 @@ display:.
 files:
     $CWD/testdata/testmod/multi2/other.cue
 imports:
-    mod.test/test/sub: $CWD/testdata/testmod/sub/sub.cue`}, {
+    mod.test/test/sub: $CWD/testdata/testmod/sub/sub.cue`,
+	}, {
 		name: "ExplicitPackageWithUnqualifiedImportPath#3",
 		cfg: &Config{
 			Dir:     filepath.Join(testdataDir, "multi3"),
@@ -482,7 +511,8 @@ display:.
 files:
     $CWD/testdata/testmod/multi3/other.cue
 imports:
-    mod.test/test/sub: $CWD/testdata/testmod/sub/sub.cue`}, {
+    mod.test/test/sub: $CWD/testdata/testmod/sub/sub.cue`,
+	}, {
 		// Test that we can explicitly ask for non-package
 		// CUE files by setting Config.Package to "_".
 		name: "ExplicitPackageWithUnqualifiedImportPath#4",
@@ -498,7 +528,8 @@ dir:    $CWD/testdata/testmod/multi4
 display:.
 files:
     $CWD/testdata/testmod/multi4/nopackage1.cue
-    $CWD/testdata/testmod/multi4/nopackage2.cue`}, {
+    $CWD/testdata/testmod/multi4/nopackage2.cue`,
+	}, {
 		// Test what happens when there's a single CUE file
 		// with an explicit `package _` directive.
 		name: "ExplicitPackageWithUnqualifiedImportPath#5",
@@ -513,7 +544,8 @@ root:   $CWD/testdata/testmod
 dir:    $CWD/testdata/testmod/multi5
 display:.
 files:
-    $CWD/testdata/testmod/multi5/nopackage.cue`}, {
+    $CWD/testdata/testmod/multi5/nopackage.cue`,
+	}, {
 		// Check that imports are only considered from files
 		// that match the build paths.
 		name: "BuildTagsWithImports#1",
@@ -531,7 +563,8 @@ files:
     $CWD/testdata/testmod/tagswithimports/prod.cue
 imports:
     mod.test/test/hello:test: $CWD/testdata/testmod/test.cue $CWD/testdata/testmod/hello/test.cue
-    mod.test/test/sub: $CWD/testdata/testmod/sub/sub.cue`}, {
+    mod.test/test/sub: $CWD/testdata/testmod/sub/sub.cue`,
+	}, {
 		// Check that imports are only considered from files
 		// that match the build paths. When we don't have the prod
 		// tag, the bad import path mentioned in testdata/testmod/tagswithimports/nonprod.cue
@@ -549,7 +582,8 @@ root:   $CWD/testdata/testmod
 dir:    $CWD/testdata/testmod/tagswithimports
 display:.
 files:
-    $CWD/testdata/testmod/tagswithimports/nonprod.cue`}, {
+    $CWD/testdata/testmod/tagswithimports/nonprod.cue`,
+	}, {
 		name: "ModuleFileNonDirectory",
 		cfg: &Config{
 			Dir: testdata("testmod_legacymodfile"),
@@ -560,7 +594,8 @@ path:   ""
 module: ""
 root:   ""
 dir:    ""
-display:""`}, {
+display:""`,
+	}, {
 		// This test checks that files in parent directories
 		// do not result in irrelevant instances appearing
 		// in the result of Instances.
@@ -611,7 +646,8 @@ dir:    $CWD/testdata/testmod/issue3306/x
 display:./issue3306/x
 files:
     $CWD/testdata/testmod/issue3306/x.cue
-    $CWD/testdata/testmod/issue3306/x/x.cue`}, {
+    $CWD/testdata/testmod/issue3306/x/x.cue`,
+	}, {
 		// This test checks that when we use Package: "*",
 		// we can still use imported packages.
 		name: "AllPackagesWithImports",
@@ -637,6 +673,20 @@ files:
     $CWD/testdata/testmod/test.cue
 imports:
     mod.test/test/sub: $CWD/testdata/testmod/sub/sub.cue`}, {
+		// This test checks that we can use a :pkg selector
+		// to select named packages from multi-package directories.
+		name: "Issue4110",
+		cfg: &Config{
+			Dir: testdataDir,
+		},
+		args: []string{"./issue3306/...:a"},
+		want: `path:   mod.test/test/issue3306/a@v0
+module: mod.test/test@v0
+root:   $CWD/testdata/testmod
+dir:    $CWD/testdata/testmod/issue3306/a
+display:./issue3306/a
+files:
+    $CWD/testdata/testmod/issue3306/a/a.cue`}, {
 		// This tests that we can load a CUE package by pointing Dir to it
 		// even when the package's directory name ends with ".cue".
 		name: "DirWithCUEFileExtension",
@@ -668,7 +718,6 @@ files:
 		got = strings.Replace(got, filepath.ToSlash(cwd), "$CWD", -1)
 		// Make test work with Windows.
 		got = strings.Replace(got, string(filepath.Separator), "/", -1)
-
 		t.Equal(got, tc.want)
 	})
 }
@@ -731,7 +780,7 @@ func TestOverlays(t *testing.T) {
 	}
 	want := []string{
 		`{msg:"Hello"}`,
-		`{a:5}`,
+		`{a:>=5&<=5}`,
 	}
 	rmSpace := func(r rune) rune {
 		if unicode.IsSpace(r) {
