@@ -18,10 +18,8 @@
 package scanner
 
 import (
-	"bytes"
 	"fmt"
 	"path/filepath"
-	"strconv"
 	"unicode"
 	"unicode/utf8"
 
@@ -48,7 +46,6 @@ type Scanner struct {
 	ch              rune // current character
 	offset          int  // character offset
 	rdOffset        int  // reading offset (position after current character)
-	lineOffset      int  // current line offset
 	linesSinceLast  int
 	spacesSinceLast int
 	insertEOL       bool // insert a comma before next newline
@@ -73,7 +70,6 @@ func (s *Scanner) next() {
 	if s.rdOffset < len(s.src) {
 		s.offset = s.rdOffset
 		if s.ch == '\n' {
-			s.lineOffset = s.offset
 			s.file.AddLine(s.offset)
 		}
 		r, w := rune(s.src[s.rdOffset]), 1
@@ -94,7 +90,6 @@ func (s *Scanner) next() {
 	} else {
 		s.offset = len(s.src)
 		if s.ch == '\n' {
-			s.lineOffset = s.offset
 			s.file.AddLine(s.offset)
 		}
 		s.ch = -1 // eof
@@ -139,7 +134,6 @@ func (s *Scanner) Init(file *token.File, src []byte, eh ErrorHandler, mode Mode)
 	s.ch = ' '
 	s.offset = 0
 	s.rdOffset = 0
-	s.lineOffset = 0
 	s.insertEOL = false
 	s.ErrorCount = 0
 
@@ -156,29 +150,6 @@ func (s *Scanner) errf(offs int, msg string, args ...interface{}) {
 	s.ErrorCount++
 }
 
-var prefix = []byte("//line ")
-
-func (s *Scanner) interpretLineComment(text []byte) {
-	if bytes.HasPrefix(text, prefix) {
-		// get filename and line number, if any
-		if i := bytes.LastIndex(text, []byte{':'}); i > 0 {
-			if line, err := strconv.Atoi(string(text[i+1:])); err == nil && line > 0 {
-				// valid //line filename:line comment
-				filename := string(bytes.TrimSpace(text[len(prefix):i]))
-				if filename != "" {
-					filename = filepath.Clean(filename)
-					if !filepath.IsAbs(filename) {
-						// make filename relative to current directory
-						filename = filepath.Join(s.dir, filename)
-					}
-				}
-				// update scanner position
-				s.file.AddLineInfo(s.lineOffset+len(text)+1, filename, line) // +len(text)+1 since comment applies to next line
-			}
-		}
-	}
-}
-
 func (s *Scanner) scanComment() string {
 	// initial '/' already consumed; s.ch == '/'
 	offs := s.offset - 1 // position of initial '/'
@@ -193,10 +164,6 @@ func (s *Scanner) scanComment() string {
 			}
 			s.next()
 		}
-		if offs == s.lineOffset {
-			// comment starts at the beginning of the current line
-			s.interpretLineComment(s.src[offs:s.offset])
-		}
 		goto exit
 	}
 
@@ -638,6 +605,11 @@ func (s *Scanner) popInterpolation() quoteInfo {
 }
 
 // ResumeInterpolation resumes scanning of a string interpolation.
+// It should be called when the final parenthesis (RPAREN) of an expression
+// inside a string interpolation has been consumed, and returns
+// the next literal segment of the interpolation string, including
+// the closing parenthesis, and possible the opening parenthesis
+// of the next interpolation expression if there is one.
 func (s *Scanner) ResumeInterpolation() string {
 	quote := s.popInterpolation()
 	_, str := s.scanString(s.offset-1, quote)
@@ -653,9 +625,10 @@ func (s *Scanner) Offset() int {
 // and its literal string if applicable. The source end is indicated by
 // EOF.
 //
-// If the returned token is a literal (IDENT, INT, FLOAT,
-// IMAG, CHAR, STRING) or COMMENT, the literal string
-// has the corresponding value.
+// If the returned token is a literal (IDENT, INT, FLOAT, IMAG, CHAR,
+// STRING, INTERPOLATION) or COMMENT or ATTRIBUTE, the literal
+// string has the corresponding value, but see below for more
+// information on INTERPOLATION.
 //
 // If the returned token is a keyword, the literal string is the keyword.
 //
@@ -679,6 +652,44 @@ func (s *Scanner) Offset() int {
 // Scan adds line information to the file added to the file
 // set with Init. Token positions are relative to that file
 // and thus relative to the file set.
+//
+// # String Interpolations
+//
+// The INTERPOLATION token is treated somewhat specially, as the scanner
+// does not itself determine the extent of the interpolation
+// expressions.
+//
+// The scanner relies on the caller to read tokens after the literal
+// string segments of the interpolation; when the caller has scanned the
+// final parenthesis, it must call [Scanner.ResumeInterpolation], which
+// returns the next segment of the interpolation, which may or may not
+// itself be an interpolation (it's an interpolation iff the final
+// character is an open-parenthesis).
+//
+// Note that the string literal associated with the INTERPOLATION token
+// and the string returned by ResumeInterpolation contain the bounding
+// parenthesis characters, even though they are also returned from the
+// scanner as separate tokens.
+//
+// For example, scanning the following CUE
+//
+//	#"a\#(foo("c \(d)"))"#
+//
+// would produce the following tokens and literal values:
+//
+//	INTERPOLATION `#"a\#(`
+//	LPAREN
+//	IDENT `b`
+//	LPAREN
+//	IDENT `foo`
+//	LPAREN
+//	INTERPOLATION `"c \(`
+//	IDENT `d`
+//	RPAREN
+//	ResumeInterpolation -> `)"`
+//	RPAREN
+//	RPAREN
+//	ResumeInterpolation -> `)"#`
 func (s *Scanner) Scan() (pos token.Pos, tok token.Token, lit string) {
 scanAgain:
 	s.skipWhitespace(1)
@@ -829,6 +840,8 @@ scanAgain:
 		case '?':
 			tok = token.OPTION
 			insertEOL = true
+		case '~':
+			tok = token.TILDE
 		case '.':
 			if '0' <= s.ch && s.ch <= '9' {
 				insertEOL = true
