@@ -50,23 +50,21 @@ var (
 //
 // Note that this also means that CUE_UPDATE=1 is broken under `go test -shuffle`.
 func TestEvalV2(t *testing.T) {
+	t.Skip("TODO: this will become the main test after EvalV3 promotes")
 	test := cuetxtar.TxTarTest{
 		Root: "../../../cue/testdata",
 		Name: "eval",
 	}
-
 	cuedebug.Init()
 	dbg := cuedebug.Flags
 	cueexperiment.Init()
 	exp := cueexperiment.Flags
-
 	if *todo {
 		test.ToDo = nil
 	}
-
 	test.Run(t, func(t *cuetxtar.Test) {
 		t.Parallel()
-		runEvalTest(t, internal.EvalV2, dbg, exp)
+		runEvalTest(t, internal.EvalV3, dbg, exp)
 	})
 }
 
@@ -121,6 +119,7 @@ func runEvalTest(t *cuetxtar.Test, version internal.EvaluatorVersion, dbg cuedeb
 		}
 		for _, f := range t.Archive.Files {
 			if f.Name != "out/eval/stats" {
+				// TODO perhaps we should also look for out/evalalpha/stats too?
 				continue
 			}
 			c := cuecontext.New()
@@ -150,6 +149,12 @@ func runEvalTest(t *cuetxtar.Test, version internal.EvaluatorVersion, dbg cuedeb
 				// solved.
 				w := t.Writer("stats")
 				fmt.Fprintln(w, counts)
+			default:
+				// This is technically a no-op but still worthwhile to
+				// inform the cuetxtar garbage collector we still care
+				// about the file.
+				w := t.Writer("stats")
+				w.Write(f.Data)
 			}
 			break
 		}
@@ -186,7 +191,6 @@ func TestIssue3985(t *testing.T) {
 	// assigned a nodeContext.
 	cuecontext.New(cuecontext.EvaluatorVersion(cuecontext.EvalV3)).CompileString(`a!: _, b: [for c in a if a != _|_ {}]`)
 
-	cuecontext.New(cuecontext.EvaluatorVersion(cuecontext.EvalV2)).CompileString(`matchN(0, [_|_]) & []`)
 }
 
 // TestX is for debugging. Do not delete.
@@ -201,14 +205,13 @@ func TestX(t *testing.T) {
 	cueexperiment.Init()
 	exps := cueexperiment.Flags
 
-	version := internal.DefaultVersion
-	version = internal.DevVersion // comment to use default implementation.
+	version := internal.EvalV3
 
 	in := `
 -- cue.mod/module.cue --
 module: "mod.test"
 
-language: version: "v0.9.0"
+language: version: "v0.15.0"
 
 -- in.cue --
 	`
