@@ -149,7 +149,7 @@ func (d *Dependency) Import() *adt.ImportReference {
 
 // IsRoot reports whether the dependency is referenced by the root of the
 // original Vertex passed to any of the Visit* functions, and not one of its
-// descendent arcs. This always returns true for Visit().
+// descendent arcs. This always returns true for [Visit].
 func (d *Dependency) IsRoot() bool {
 	return d.top
 }
@@ -196,6 +196,7 @@ func Visit(cfg *Config, c *adt.OpContext, n *adt.Vertex, f VisitFunc) error {
 		all:        cfg.Descend,
 		top:        true,
 		cfgDynamic: cfg.Dynamic,
+		resolved:   map[refEntry]bool{},
 	}
 	return v.visitReusingVisitor(n, true)
 }
@@ -235,10 +236,9 @@ func (v *visitor) visit(n *adt.Vertex, top bool) (err error) {
 		}
 	}()
 
-	n.VisitLeafConjuncts(func(x adt.Conjunct) bool {
+	for x := range n.LeafConjuncts() {
 		v.markExpr(x.Env, x.Elem())
-		return true
-	})
+	}
 
 	return nil
 }
@@ -268,6 +268,9 @@ type visitor struct {
 	cfgDynamic bool
 
 	marked marked
+
+	// resolved dedups resolving references to prevent exponential blowup.
+	resolved map[refEntry]bool
 }
 
 type refEntry struct {
@@ -298,6 +301,9 @@ func (c *visitor) markExpr(env *adt.Environment, expr adt.Elem) {
 	case *adt.UnaryExpr:
 		c.markExpr(env, x.X)
 
+	case *adt.OpenExpr:
+		c.markExpr(env, x.X)
+
 	case *adt.Interpolation:
 		for i := 1; i < len(x.Parts); i += 2 {
 			c.markExpr(env, x.Parts[i])
@@ -356,10 +362,18 @@ func (c *visitor) markExpr(env *adt.Environment, expr adt.Elem) {
 
 // markResolve resolves dependencies.
 func (c *visitor) markResolver(env *adt.Environment, r adt.Resolver) {
+	if c.resolved[refEntry{env, r}] {
+		// TODO: this seems to still not remove everything. Consider a
+		// different approach.
+		return
+	}
+
 	// Note: it is okay to pass an empty CloseInfo{} here as we assume that
 	// all nodes are finalized already and we need neither closedness nor cycle
 	// checks.
 	ref, _ := c.ctxt.Resolve(adt.MakeConjunct(env, r, adt.CloseInfo{}), r)
+	c.resolved[refEntry{env, r}] = true
+	c.ctxt.Stats().ResolveDep++
 
 	// TODO: consider the case where an inlined composite literal does not
 	// resolve, but has references. For instance, {a: k, ref}.b would result
@@ -413,15 +427,31 @@ func (c *visitor) reportDependency(env *adt.Environment, ref adt.Resolver, v *ad
 		reference = c.topRef
 	}
 
-	if !v.Rooted() {
+	inspect := v.IsDetached() || !v.MayAttach()
+
+	if inspect {
+		// TODO: there is currently no way to inspect where a non-rooted node
+		// originated from. As of EvalV3, we allow non-rooted nodes to be
+		// structure shared. This makes them effectively rooted, with the
+		// difference that there is an indirection in BaseValue for the
+		// structure sharing. Nonetheless, this information is lost in the
+		// internal API when traversing.
+
+		// As an alternative we now do not skip processing the node if we
+		// an inlined, non-rooted node is associated with another node than
+		// the one we are currently processing.
+
+		// If a node is internal, we need to further investigate any references.
+		// If there are any, reference, even if it is otherwise not reported,
+		// we report this reference.
 		before := c.numRefs
 		c.markInternalResolvers(env, ref, v)
 		// TODO: this logic could probably be simplified if we let clients
 		// explicitly mark whether to visit rootless nodes. Visiting these
 		// may be necessary when substituting values.
 		switch _, ok := ref.(*adt.FieldReference); {
-		case !ok:
-			// Do not report rootless nodes for selectors.
+		case !ok && c.isLocal(env, ref):
+			// 	Do not report rootless nodes for selectors.
 			return
 		case c.numRefs > before:
 			// For FieldReferences that resolve to something we do not need
@@ -453,6 +483,9 @@ func (c *visitor) reportDependency(env *adt.Environment, ref adt.Resolver, v *ad
 		}
 		v = w
 	}
+	if inspect && len(c.pathStack) == 0 && c.topRef != nil {
+		altRef = c.topRef
+	}
 
 	// All resolvers are expressions.
 	if p := importRef(ref.(adt.Expr)); p != nil {
@@ -463,6 +496,9 @@ func (c *visitor) reportDependency(env *adt.Environment, ref adt.Resolver, v *ad
 
 	c.numRefs++
 
+	// Note: we did not finalize in V2.
+	v.Unify(c.ctxt, adt.FinalizeWithoutTypoCheck)
+
 	d := Dependency{
 		Node:      v,
 		Reference: altRef,
@@ -476,6 +512,26 @@ func (c *visitor) reportDependency(env *adt.Environment, ref adt.Resolver, v *ad
 	}
 }
 
+// isLocal reports whether a non-rooted struct is an internal node or not.
+// If it is not, we need to further investigate any references.
+func (c *visitor) isLocal(env *adt.Environment, r adt.Resolver) bool {
+	for {
+		switch x := r.(type) {
+		case *adt.FieldReference:
+			for i := 0; i < int(x.UpCount); i++ {
+				env = env.Up
+			}
+			return env.Vertex == empty
+		case *adt.SelectorExpr:
+			r, _ = x.X.(adt.Resolver)
+		case *adt.IndexExpr:
+			r, _ = x.X.(adt.Resolver)
+		default:
+			return env.Vertex == empty
+		}
+	}
+}
+
 // TODO(perf): make this available as a property of vertices to avoid doing
 // work repeatedly.
 func hasLetParent(v *adt.Vertex) bool {
@@ -489,12 +545,11 @@ func hasLetParent(v *adt.Vertex) bool {
 
 // markConjuncts transitively marks all reference of the current node.
 func (c *visitor) markConjuncts(v *adt.Vertex) {
-	v.VisitLeafConjuncts(func(x adt.Conjunct) bool {
+	for x := range v.LeafConjuncts() {
 		// Use Elem instead of Expr to preserve the Comprehension to, in turn,
 		// ensure an Environment is inserted for the Value clause.
 		c.markExpr(x.Env, x.Elem())
-		return true
-	})
+	}
 }
 
 // markInternalResolvers marks dependencies for rootless nodes. As these
@@ -502,19 +557,14 @@ func (c *visitor) markConjuncts(v *adt.Vertex) {
 // proactive. For selectors and indices this means we need to evaluate their
 // objects to see exactly what the selector or index refers to.
 func (c *visitor) markInternalResolvers(env *adt.Environment, r adt.Resolver, v *adt.Vertex) {
-	if v.Rooted() {
-		panic("node must not be rooted")
-	}
-
 	saved := c.all // recursive traversal already done by this function.
 
 	// As lets have no path and we otherwise will not process them, we set
 	// processing all to true.
 	if c.marked != nil && hasLetParent(v) {
-		v.VisitLeafConjuncts(func(x adt.Conjunct) bool {
+		for x := range v.LeafConjuncts() {
 			c.marked.markExpr(x.Expr())
-			return true
-		})
+		}
 	}
 
 	c.markConjuncts(v)
@@ -544,7 +594,7 @@ func (c *visitor) evaluateInner(env *adt.Environment, x adt.Expr, r adt.Resolver
 		return
 	}
 	// TODO(perf): one level of  evaluation would suffice.
-	v.Finalize(c.ctxt)
+	v.Unify(c.ctxt, adt.FinalizeWithoutTypoCheck)
 
 	saved := len(c.pathStack)
 	c.pathStack = append(c.pathStack, refEntry{env, r})
