@@ -22,12 +22,14 @@ import (
 	"text/tabwriter"
 
 	"cuelang.org/go/cue"
-	"cuelang.org/go/cue/cuecontext"
 	"cuelang.org/go/cue/format"
+	"cuelang.org/go/internal"
 	"cuelang.org/go/internal/core/adt"
 	"cuelang.org/go/internal/core/debug"
 	"cuelang.org/go/internal/core/dep"
 	"cuelang.org/go/internal/core/eval"
+	"cuelang.org/go/internal/core/runtime"
+	"cuelang.org/go/internal/cuedebug"
 	"cuelang.org/go/internal/cuetdtest"
 	"cuelang.org/go/internal/cuetxtar"
 	"cuelang.org/go/internal/value"
@@ -45,7 +47,15 @@ func TestVisit(t *testing.T) {
 	}
 
 	test.Run(t, func(t *cuetxtar.Test) {
-		val := t.Context().BuildInstance(t.Instance())
+		flags := cuedebug.Config{
+			Sharing: true,
+			// LogEval: 1,
+		}
+		ctx := t.CueContext()
+		r := (*runtime.Runtime)(ctx)
+		r.SetDebugOptions(&flags)
+
+		val := ctx.BuildInstance(t.Instance())
 		if val.Err() != nil {
 			t.Fatal(val.Err())
 		}
@@ -76,8 +86,12 @@ func TestVisit(t *testing.T) {
 			_, n := value.ToInternal(v)
 			w := t.Writer(tc.name)
 
+			stats := t.Writer("stats/" + tc.name)
+
 			t.Run(tc.name, func(sub *testing.T) {
 				testVisit(sub, w, ctxt, n, tc.cfg)
+				fmt.Fprintf(stats, "// COUNT: %d\n", ctxt.Stats().ResolveDep)
+
 			})
 		}
 	})
@@ -105,7 +119,7 @@ func testVisit(t *testing.T, w io.Writer, ctxt *adt.OpContext, v *adt.Vertex, cf
 		if i := d.Import(); i != nil {
 			path := i.ImportPath.StringValue(ctxt)
 			str = fmt.Sprintf("%q.%s", path, str)
-		} else if !d.Node.Rooted() {
+		} else if d.Node.IsDetached() {
 			str = "**non-rooted**"
 		}
 
@@ -117,9 +131,15 @@ func testVisit(t *testing.T, w io.Writer, ctxt *adt.OpContext, v *adt.Vertex, cf
 
 // DO NOT REMOVE: for Testing purposes.
 func TestX(t *testing.T) {
+	version := internal.EvalV3
+	flags := cuedebug.Config{
+		Sharing: true,
+		LogEval: 1,
+	}
+
 	cfg := &dep.Config{
 		Dynamic: true,
-		// Recurse: true,
+		// Descend: true,
 	}
 
 	in := `
@@ -129,7 +149,10 @@ func TestX(t *testing.T) {
 		t.Skip()
 	}
 
-	v := cuecontext.New().CompileString(in)
+	r := runtime.NewWithSettings(version, flags)
+	ctx := (*cue.Context)(r)
+
+	v := ctx.CompileString(in)
 	if err := v.Err(); err != nil {
 		t.Fatal(err)
 	}
@@ -138,13 +161,15 @@ func TestX(t *testing.T) {
 
 	r, n := value.ToInternal(aVal)
 
+	out := debug.NodeString(r, n, nil)
+	t.Error(out)
+
 	ctxt := eval.NewContext(r, n)
 
-	n.VisitLeafConjuncts(func(c adt.Conjunct) bool {
+	for c := range n.LeafConjuncts() {
 		str := debug.NodeString(ctxt, c.Elem(), nil)
 		t.Log(str)
-		return true
-	})
+	}
 
 	w := &strings.Builder{}
 	fmt.Fprintln(w)
@@ -152,4 +177,5 @@ func TestX(t *testing.T) {
 	testVisit(t, w, ctxt, n, cfg)
 
 	t.Error(w.String())
+	t.Error("COUNT", ctxt.Stats().ResolveDep)
 }
