@@ -16,7 +16,9 @@ package export
 
 import (
 	"bytes"
+	"cmp"
 	"fmt"
+	"slices"
 	"strings"
 
 	"cuelang.org/go/cue/ast"
@@ -53,7 +55,6 @@ func (e *exporter) adt(env *adt.Environment, expr adt.Elem) ast.Expr {
 		// _, saved := e.pushFrame([]adt.Conjunct{adt.MakeConjunct(nil, x)})
 		// defer e.popFrame(saved)
 		// s := e.frame(0).scope
-
 		s := &ast.StructLit{}
 		// TODO: ensure e.node() is set in more cases. Right now it is not
 		// always set in mergeValues, even in cases where it could be. Better
@@ -229,7 +230,16 @@ func (e *exporter) adt(env *adt.Environment, expr adt.Elem) ast.Expr {
 			X:  e.innerExpr(env, x.X),
 		}
 
+	case *adt.OpenExpr:
+		return &ast.PostfixExpr{
+			X:  e.innerExpr(env, x.X),
+			Op: token.ELLIPSIS,
+		}
+
 	case *adt.BinaryExpr:
+		if x.Op == adt.AndOp || x.Op == adt.OrOp {
+			return e.sortBinaryTree(env, x)
+		}
 		return &ast.BinaryExpr{
 			Op: x.Op.Token(),
 			X:  e.innerExpr(env, x.X),
@@ -263,7 +273,7 @@ func (e *exporter) adt(env *adt.Environment, expr adt.Elem) ast.Expr {
 	case *adt.ConjunctGroup:
 		a := []ast.Expr{}
 		for _, c := range *x {
-			v := e.expr(env, c.Expr())
+			v := e.expr(c.EnvExpr())
 			a = append(a, v)
 		}
 		return ast.NewBinExpr(token.AND, a...)
@@ -293,6 +303,7 @@ func (e *exporter) adt(env *adt.Environment, expr adt.Elem) ast.Expr {
 			env = &adt.Environment{Up: env, Vertex: empty}
 		}
 
+		// TODO: consider using adt.EnvExpr.
 		return e.adt(env, adt.ToExpr(x.Value))
 
 	default:
@@ -300,6 +311,89 @@ func (e *exporter) adt(env *adt.Environment, expr adt.Elem) ast.Expr {
 	}
 }
 
+// sortBinaryTree converte x to a binary tree and sorts it's elements
+// using sortLeafAdt.
+func (e *exporter) sortBinaryTree(env *adt.Environment, x *adt.BinaryExpr) (b ast.Expr) {
+	var exprs []adt.Node
+
+	var flatten func(expr adt.Expr)
+	flatten = func(expr adt.Expr) {
+		if y, ok := expr.(*adt.BinaryExpr); ok && x.Op == y.Op {
+			flatten(y.X)
+			flatten(y.Y)
+		} else {
+			exprs = append(exprs, expr)
+		}
+	}
+	flatten(x)
+
+	// Sort the expressions
+	slices.SortStableFunc(exprs, cmpLeafNodes)
+
+	nodes := make([]ast.Expr, 0, len(exprs))
+	for _, x := range exprs {
+		switch y := x.(type) {
+		case *adt.Top:
+		case *adt.BasicType:
+			if y.K != adt.TopKind {
+				nodes = append(nodes, e.expr(env, y))
+			}
+		default:
+			nodes = append(nodes, e.innerExpr(env, y.(adt.Expr)))
+		}
+	}
+
+	if len(nodes) == 0 {
+		return e.adt(env, &adt.Top{})
+	}
+
+	return ast.NewBinExpr(x.Op.Token(), nodes...)
+}
+
+// cmpConjuncts compares two Conjunct based on their element using cmpLeafNodes.
+func cmpConjuncts(a, b adt.Conjunct) int {
+	return cmpLeafNodes(a.Expr(), b.Expr())
+}
+
+// cmpLeafNodes compares two adt.Expr values. The values may not be a binary
+// expressions. It returns true if a is less than b.
+func cmpLeafNodes[T adt.Node](a, b T) int {
+	if c := cmp.Compare(typeOrder(a), typeOrder(b)); c != 0 {
+		return c
+	}
+
+	srcA := a.Source()
+	srcB := b.Source()
+
+	if srcA == nil || srcB == nil {
+		// TODO: some tie breaker
+		return 0
+	}
+
+	return srcA.Pos().Compare(srcB.Pos())
+}
+
+func typeOrder(x adt.Node) int {
+	switch x.(type) {
+	case *adt.Top:
+		return 0
+	case *adt.BasicType:
+		return 1
+	case *adt.FieldReference:
+		return 2 // sometimes basic types are represented as field references.
+	case *adt.Bool, *adt.Null, *adt.Num, *adt.String, *adt.Bytes:
+		return 10
+	case *adt.BoundValue:
+		return 20
+	case *adt.StructLit, *adt.ListLit:
+		return 500
+	case adt.Expr:
+		return 25
+	default:
+		return 100
+	}
+}
+
 var dummyTop = &ast.Ident{Name: "_"}
 
 func (e *exporter) resolve(env *adt.Environment, r adt.Resolver) ast.Expr {
@@ -395,13 +489,13 @@ func (e *exporter) resolve(env *adt.Environment, r adt.Resolver) ast.Expr {
 
 	case *adt.ImportReference:
 		importPath := x.ImportPath.StringValue(e.index)
+		info := ast.ParseImportPath(importPath)
 		spec := ast.NewImport(nil, importPath)
 
-		info, _ := astutil.ParseImportSpec(spec)
-		name := info.PkgName
+		name := info.Qualifier
 		if x.Label != 0 {
-			name = x.Label.StringValue(e.index)
-			if name != info.PkgName {
+			name = x.Label.IdentString(e.index)
+			if name != info.Qualifier {
 				spec.Name = ast.NewIdent(name)
 			}
 		}
@@ -704,6 +798,7 @@ func (e *exporter) comprehension(env *adt.Environment, comp *adt.Comprehension)
 		env = &adt.Environment{Up: env, Vertex: empty}
 	}
 
+	// TODO: consider using adt.EnvExpr.
 	v := e.expr(env, adt.ToExpr(comp.Value))
 	if _, ok := v.(*ast.StructLit); !ok {
 		v = ast.NewStruct(ast.Embed(v))
