@@ -209,23 +209,6 @@ func makeFeatures(index adt.StringIndexer, inputs [][]string) [][]adt.Feature {
 	return result
 }
 
-// Consider that names are nodes in a cycle, we want to rotate the
-// slice so that it starts at the given node name. This modifies the
-// names slice in-place.
-func rotateToStartAt(names []string, start string) {
-	if start == names[0] {
-		return
-	}
-	for i, node := range names {
-		if start == node {
-			prefix := slices.Clone(names[:i])
-			copy(names, names[i:])
-			copy(names[len(names)-i:], prefix)
-			break
-		}
-	}
-}
-
 func allPermutations(featureses [][]adt.Feature) [][][]adt.Feature {
 	nonNilIdx := -1
 	var results [][][]adt.Feature
