@@ -27,13 +27,14 @@ import (
 )
 
 var parseTests = []struct {
-	testName     string
-	parse        func(modfile []byte, filename string) (*File, error)
-	data         string
-	wantError    string
-	want         *File
-	wantVersions []module.Version
-	wantDefaults map[string]string
+	testName             string
+	parse                func(modfile []byte, filename string) (*File, error)
+	data                 string
+	wantError            string
+	want                 *File
+	wantVersions         []module.Version
+	wantDefaults         map[string]string
+	wantModVersionForPkg map[string]string
 }{{
 	testName: "NoDeps",
 	parse:    Parse,
@@ -50,6 +51,15 @@ language: version: "v0.8.0-alpha.0"
 	wantDefaults: map[string]string{
 		"foo.com/bar": "v0",
 	},
+	wantModVersionForPkg: map[string]string{
+		"foo.com/bar":              "foo.com/bar@v0",
+		"foo.com/bar@v0":           "foo.com/bar@v0",
+		"foo.com/bar/baz@v0":       "foo.com/bar@v0",
+		"foo.com/bar@v1":           "",
+		"foo.com/bar:hello":        "foo.com/bar@v0",
+		"foo.com/bar/baz:hello":    "foo.com/bar@v0",
+		"foo.com/bar/baz@v0:hello": "foo.com/bar@v0",
+	},
 }, {
 	testName: "WithDeps",
 	parse:    Parse,
@@ -60,6 +70,11 @@ deps: "example.com@v1": {
 	default: true
 	v: "v1.2.3"
 }
+deps: "example.com/other@v1": v: "v1.9.10"
+deps: "example.com/other/more/nested@v2": {
+	v: "v2.9.20"
+	default: true
+}
 deps: "other.com/something@v0": v: "v0.2.3"
 `,
 	want: &File{
@@ -75,12 +90,36 @@ deps: "other.com/something@v0": v: "v0.2.3"
 			"other.com/something@v0": {
 				Version: "v0.2.3",
 			},
+			"example.com/other@v1": {
+				Version: "v1.9.10",
+			},
+			"example.com/other/more/nested@v2": {
+				Version: "v2.9.20",
+				Default: true,
+			},
 		},
 	},
-	wantVersions: parseVersions("example.com@v1.2.3", "other.com/something@v0.2.3"),
+	wantVersions: parseVersions(
+		"example.com/other/more/nested@v2.9.20",
+		"example.com/other@v1.9.10",
+		"example.com@v1.2.3",
+		"other.com/something@v0.2.3",
+	),
 	wantDefaults: map[string]string{
-		"foo.com/bar": "v0",
-		"example.com": "v1",
+		"example.com/other/more/nested": "v2",
+		"foo.com/bar":                   "v0",
+		"example.com":                   "v1",
+	},
+	wantModVersionForPkg: map[string]string{
+		"example.com":                       "example.com@v1.2.3",
+		"example.com/x/y@v1":                "example.com@v1.2.3",
+		"example.com/x/y@v1:x":              "example.com@v1.2.3",
+		"example.com/other@v1":              "example.com/other@v1.9.10",
+		"example.com/other/p@v1":            "example.com/other@v1.9.10",
+		"example.com/other/more":            "example.com@v1.2.3",
+		"example.com/other/more@v1":         "example.com/other@v1.9.10",
+		"example.com/other/more/nested":     "example.com/other/more/nested@v2.9.20",
+		"example.com/other/more/nested/x:p": "example.com/other/more/nested@v2.9.20",
 	},
 }, {
 	testName: "WithSource",
@@ -139,7 +178,7 @@ module: "foo.com/bar@v0"
 language: version: "v0.8.6"
 source: kind: "git"
 `,
-	wantError: `invalid module.cue file: source field is not allowed at this language version; need at least v0.9.0-alpha.0`,
+	wantError: `invalid module file: source field is not allowed at this language version; need at least v0.9.0-alpha.0`,
 }, {
 	testName: "AmbiguousDefaults",
 	parse:    Parse,
@@ -155,7 +194,7 @@ deps: "example.com@v2": {
 	v: "v2.0.0"
 }
 `,
-	wantError: `multiple default major versions found for example.com`,
+	wantError: `invalid module file module.cue: multiple default major versions found for example.com`,
 }, {
 	testName: "AmbiguousDefaultsWithMainModule",
 	parse:    Parse,
@@ -167,7 +206,7 @@ deps: "foo.com/bar@v1": {
 	v: "v1.2.3"
 }
 `,
-	wantError: `multiple default major versions found for foo.com/bar`,
+	wantError: `invalid module file module.cue: multiple default major versions found for foo.com/bar`,
 }, {
 	testName: "MisspelledLanguageVersionField",
 	parse:    Parse,
@@ -204,7 +243,7 @@ module: "foo.com/bar@v0"`,
 module: "foo.com/bar@v0"
 language: version: "v0.8"
 `,
-	wantError: `language version v0.8 in module.cue is not canonical`,
+	wantError: `invalid module file module.cue: language version v0.8 is not canonical`,
 }, {
 	testName: "InvalidDepVersion",
 	parse:    Parse,
@@ -213,7 +252,7 @@ module: "foo.com/bar@v1"
 language: version: "v0.8.0"
 deps: "example.com@v1": v: "1.2.3"
 `,
-	wantError: `invalid module.cue file module.cue: cannot make version from module "example.com@v1", version "1.2.3": version "1.2.3" \(of module "example.com@v1"\) is not well formed`,
+	wantError: `invalid module file module.cue: cannot make version from module "example.com@v1", version "1.2.3": version "1.2.3" \(of module "example.com@v1"\) is not well formed`,
 }, {
 	testName: "NonCanonicalVersion",
 	parse:    Parse,
@@ -222,7 +261,7 @@ module: "foo.com/bar@v1"
 language: version: "v0.8.0"
 deps: "example.com@v1": v: "v1.2"
 `,
-	wantError: `invalid module.cue file module.cue: cannot make version from module "example.com@v1", version "v1.2": version "v1.2" \(of module "example.com@v1"\) is not canonical`,
+	wantError: `invalid module file module.cue: cannot make version from module "example.com@v1", version "v1.2": version "v1.2" \(of module "example.com@v1"\) is not canonical`,
 }, {
 	testName: "NonCanonicalModule",
 	parse:    Parse,
@@ -230,7 +269,7 @@ deps: "example.com@v1": v: "v1.2"
 module: "foo.com/bar@v0.1.2"
 language: version: "v0.8.0"
 `,
-	wantError: `module path foo.com/bar@v0.1.2 in "module.cue" should contain the major version only`,
+	wantError: `invalid module file module.cue: module path foo.com/bar@v0.1.2 should contain the major version only`,
 }, {
 	testName: "NonCanonicalDep",
 	parse:    Parse,
@@ -239,7 +278,7 @@ module: "foo.com/bar@v1"
 language: version: "v0.8.0"
 deps: "example.com": v: "v1.2.3"
 `,
-	wantError: `invalid module.cue file module.cue: no major version in "example.com"`,
+	wantError: `invalid module file module.cue: no major version in "example.com"`,
 }, {
 	testName: "MismatchedMajorVersion",
 	parse:    Parse,
@@ -248,7 +287,7 @@ module: "foo.com/bar@v1"
 language: version: "v0.8.0"
 deps: "example.com@v1": v: "v0.1.2"
 `,
-	wantError: `invalid module.cue file module.cue: cannot make version from module "example.com@v1", version "v0.1.2": mismatched major version suffix in "example.com@v1" \(version v0.1.2\)`,
+	wantError: `invalid module file module.cue: cannot make version from module "example.com@v1", version "v0.1.2": mismatched major version suffix in "example.com@v1" \(version v0.1.2\)`,
 }, {
 	testName: "NonStrictNoMajorVersions",
 	parse:    ParseNonStrict,
@@ -270,6 +309,12 @@ deps: "example.com": v: "v1.2.3"
 	wantDefaults: map[string]string{
 		"foo.com/bar": "v0",
 	},
+	wantModVersionForPkg: map[string]string{
+		"example.com":          "", // No default major version.
+		"example.com@v1":       "example.com@v1.2.3",
+		"example.com/x/y@v1":   "example.com@v1.2.3",
+		"example.com/x/y@v1:x": "example.com@v1.2.3",
+	},
 }, {
 	testName: "LegacyWithExtraFields",
 	parse:    ParseLegacy,
@@ -288,7 +333,7 @@ language: version: "xxx"
 module: _foo
 _foo: "blah.example"
 `,
-	wantError: `invalid module.cue file syntax: references not allowed in data mode:
+	wantError: `invalid module file syntax: references not allowed in data mode:
     module.cue:2:9`,
 }, {
 	testName: "LegacyNoModule",
@@ -300,6 +345,11 @@ _foo: "blah.example"
 	parse:    ParseLegacy,
 	data:     `module: ""`,
 	want:     &File{},
+}, {
+	testName:  "NonLegacyEmptyModule",
+	parse:     Parse,
+	data:      `module: "", language: version: "v0.8.0"`,
+	wantError: `invalid module file module.cue: empty module path`,
 }, {
 	testName: "ReferencesNotAllowed#1",
 	parse:    Parse,
@@ -308,7 +358,7 @@ module: "foo.com/bar"
 _foo: "v0.9.0"
 language: version: _foo
 `,
-	wantError: `invalid module.cue file syntax: references not allowed in data mode:
+	wantError: `invalid module file syntax: references not allowed in data mode:
     module.cue:4:20`,
 }, {
 	testName: "ReferencesNotAllowed#2",
@@ -318,9 +368,9 @@ module: "foo.com/bar"
 let foo = "v0.9.0"
 language: version: foo
 `,
-	wantError: `invalid module.cue file syntax: references not allowed in data mode:
+	wantError: `invalid module file syntax: references not allowed in data mode:
     module.cue:3:1
-invalid module.cue file syntax: references not allowed in data mode:
+invalid module file syntax: references not allowed in data mode:
     module.cue:4:20`,
 }, {
 	testName: "DefinitionsNotAllowed",
@@ -330,7 +380,7 @@ module: "foo.com/bar"
 #x: "v0.9.0"
 language: version: "v0.9.0"
 `,
-	wantError: `invalid module.cue file syntax: definitions not allowed in data mode:
+	wantError: `invalid module file syntax: definitions not allowed in data mode:
     module.cue:3:1`,
 }, {
 	testName: "CustomData",
@@ -404,7 +454,7 @@ other: field: 123
 		Custom: map[string]map[string]any{
 			"legacy": {
 				"some":  true,
-				"other": map[string]any{"field": 123},
+				"other": map[string]any{"field": int64(123)},
 			},
 		},
 	},
@@ -429,17 +479,28 @@ func TestParse(t *testing.T) {
 			path, vers, ok := strings.Cut(f.Module, "@")
 			if ok {
 				qt.Assert(t, qt.Equals(f.QualifiedModule(), f.Module))
-				qt.Assert(t, qt.Equals(f.ModulePath(), path))
+				qt.Assert(t, qt.Equals(f.ModuleRootPath(), path))
 				qt.Assert(t, qt.Equals(f.MajorVersion(), vers))
 			} else if f.Module == "" {
 				qt.Assert(t, qt.Equals(f.QualifiedModule(), ""))
-				qt.Assert(t, qt.Equals(f.ModulePath(), ""))
+				qt.Assert(t, qt.Equals(f.ModuleRootPath(), ""))
 				qt.Assert(t, qt.Equals(f.MajorVersion(), ""))
 			} else {
 				qt.Assert(t, qt.Equals(f.QualifiedModule(), f.Module+"@v0"))
-				qt.Assert(t, qt.Equals(f.ModulePath(), f.Module))
+				qt.Assert(t, qt.Equals(f.ModuleRootPath(), f.Module))
 				qt.Assert(t, qt.Equals(f.MajorVersion(), "v0"))
 			}
+			for p, m := range test.wantModVersionForPkg {
+				t.Run("package-"+p, func(t *testing.T) {
+					mv, ok := f.ModuleForImportPath(p)
+					if m == "" {
+						qt.Assert(t, qt.IsFalse(ok), qt.Commentf("got version %v", mv))
+						return
+					}
+					qt.Check(t, qt.IsTrue(ok))
+					qt.Check(t, qt.Equals(mv.String(), m))
+				})
+			}
 		})
 	}
 }
@@ -479,7 +540,8 @@ deps: {
 		v: "v0.2.3"
 	}
 }
-`}, {
+`,
+	}, {
 		name: "WithoutLanguage",
 		file: &File{
 			Module: "foo.com/bar@v0",
@@ -491,7 +553,8 @@ deps: {
 language: {
 	version: "v0.8.0"
 }
-`}, {
+`,
+	}, {
 		name: "WithVersionTooEarly",
 		file: &File{
 			Module: "foo.com/bar@v0",
@@ -525,7 +588,7 @@ language: {
 `,
 	}}
 	cuetest.Run(t, tests, func(t *cuetest.T, test *formatTest) {
-		data, err := test.file.Format()
+		data, err := Format(test.file)
 		if test.wantError != "" {
 			qt.Assert(t, qt.ErrorMatches(err, test.wantError))
 			return
